@@ -34,12 +34,64 @@ static gfloat amount_first_r[2] = {0.0,1.0} ;
 static gfloat amount_last_r[2] = {0.0,1.0} ;
 static int feather_width = 20 ;
 
+void simple_amplify_audio(struct sound_prefs *p, long first, long last, int channel_mask, double amount)
+{
+    long left[BUFSIZE], right[BUFSIZE] ;
+    long current, i ;
+    int loops = 0 ;
+
+    current = first ;
+
+    push_status_text("Amplifying audio") ;
+    update_status_bar(0.0,STATUS_UPDATE_INTERVAL,TRUE) ;
+
+    {
+
+	while(current <= last) {
+	    long n = MIN(last - current + 1, BUFSIZE) ;
+	    long tmplast = current + n - 1 ;
+	    gfloat p = (gfloat)(current-first)/(last-first+1) ;
+
+	    n = read_wavefile_data(left, right, current, tmplast) ;
+
+	    update_status_bar(p,STATUS_UPDATE_INTERVAL,FALSE) ;
+
+	    for(i = 0 ; i < n ; i++) {
+		if(channel_mask & 0x01) {
+		    left[i] = lrint(amount*left[i]) ;
+		}
+
+		if(channel_mask & 0x02) {
+		    right[i] = lrint(amount*right[i]) ;
+
+		}
+	    }
+
+	    write_wavefile_data(left, right, current, tmplast) ;
+
+	    current += n ;
+
+	    if(last - current < 10) loops++ ;
+
+	    if(loops > 5) {
+		warning("inifinite loop in amplify_audio, programming error\n") ;
+	    }
+	}
+
+	resample_audio_data(p, first, last) ;
+	save_sample_block_data(p) ;
+    }
+
+    update_status_bar(0.0,STATUS_UPDATE_INTERVAL,TRUE) ;
+    pop_status_text() ;
+
+    main_redraw(FALSE, TRUE) ;
+}
+
 
 void amplify_audio(struct sound_prefs *p, long first, long last, int channel_mask)
 {
     long left[BUFSIZE], right[BUFSIZE] ;
-    long x_left[3], x_right[3] ;
-    long y_left[3], y_right[3] ;
     long current, i ;
     int loops = 0 ;
 
@@ -61,41 +113,34 @@ void amplify_audio(struct sound_prefs *p, long first, long last, int channel_mas
 
 	    for(i = 0 ; i < n ; i++) {
 		long icurrent = current + i ;
-		double p_last = (gfloat)(icurrent-first)/(last-first+1) ;
+		double p_last = (double)(icurrent-first+1)/(double)(last-first+1) ;
 		double p_first = 1.0 - p_last ;
 		double feather_p = 1.0 ;
 		double wet_left, wet_right ;
 
-		x_left[0] = x_left[1] ;
-		x_left[1] = x_left[2] ;
-
-		x_right[0] = x_right[1] ;
-		x_right[1] = x_right[2] ;
-
-		y_left[0] = y_left[1] ;
-		y_left[1] = y_left[2] ;
-
-		y_right[0] = y_right[1] ;
-		y_right[1] = y_right[2] ;
-		x_right[2] = right[i] ;
-
 		if(icurrent - first < feather_width)
-			feather_p = (double)(icurrent-first)/(feather_width) ;
+			feather_p = (double)(icurrent-first)/(double)(feather_width) ;
 
 		if(last - icurrent < feather_width)
-			feather_p = (double)(last - icurrent)/(feather_width) ;
+			feather_p = (double)(last - icurrent)/(double)(feather_width) ;
 
 		if(channel_mask & 0x01) {
-		    wet_left = (left[i]*(amount_first_l[0]*p_first+amount_last_l[0]*p_last)) ;
-		    wet_left += (right[i]*(amount_first_r[0]*p_first+amount_last_r[0]*p_last)) ;
-		    left[i] = lrint(left[i]*(1.0-feather_p) + wet_left*feather_p) ;
+		    wet_left = ((double)left[i]*(amount_first_l[0]*p_first+amount_last_l[0]*p_last)) ;
+		    wet_left += ((double)right[i]*(amount_first_r[0]*p_first+amount_last_r[0]*p_last)) ;
+		}
+
+
+		if(channel_mask & 0x02) {
+		    wet_right = ((double)left[i]*(amount_first_l[1]*p_first+amount_last_l[1]*p_last)) ;
+		    wet_right += ((double)right[i]*(amount_first_r[1]*p_first+amount_last_r[1]*p_last)) ;
+		}
+
+		if(channel_mask & 0x01) {
+		    left[i] = lrint((double)left[i]*(1.0-feather_p) + wet_left*feather_p) ;
 		}
 
 		if(channel_mask & 0x02) {
-		    wet_right = (left[i]*(amount_first_l[1]*p_first+amount_last_l[1]*p_last)) ;
-		    wet_right += (right[i]*(amount_first_r[1]*p_first+amount_last_r[1]*p_last)) ;
-		    right[i] = lrint(right[i]*(1.0-feather_p) + wet_right*feather_p) ;
-
+		    right[i] = lrint((double)right[i]*(1.0-feather_p) + wet_right*feather_p) ;
 		}
 	    }
 
