@@ -253,10 +253,12 @@ long start_playback(char *output_device, struct view *v, struct sound_prefs *p,
 
     if (audio_device_open(output_device) == -1) {
 	char buf[255] ;
+	snprintf(buf, sizeof(buf), "Failed to open OSS audio output device %s, check settings->miscellaneous for device information", output_device) ;
 #ifdef HAVE_ALSA
 	snprintf(buf, sizeof(buf), "Failed to open alsa output device %s, check settings->miscellaneous for device information", output_device) ;
-#else
-	snprintf(buf, sizeof(buf), "Failed to open output device %s, check settings->miscellaneous for device information", output_device) ;
+#endif
+#ifdef HAVE_PULSE_AUDIO
+	snprintf(buf, sizeof(buf), "Failed to open Pulse audio output device, recommend internet search about pulse audio configuration for your OS") ;
 #endif
 	warning(buf) ;
 	return 0 ;
