@@ -54,6 +54,9 @@ int row2filter(int row)
     if(row == 1) return HPF ;
     if(row == 2) return NOTCH ;
     if(row == 3) return BPF ;
+    if(row == 4) return PEQ ;
+    if(row == 5) return LSH ;
+    if(row == 6) return HSH ;
     return 0 ;
 }
 
@@ -63,6 +66,9 @@ int filter2row(gint filter_type)
     if(filter_type == HPF) return 1 ;
     if(filter_type == NOTCH) return 2 ;
     if(filter_type == BPF) return 3 ;
+    if(filter_type == PEQ) return 4 ;
+    if(filter_type == LSH) return 5 ;
+    if(filter_type == HSH) return 6 ;
     return 0 ;
 }
 
@@ -109,7 +115,6 @@ void filter_audio(struct sound_prefs *p, long first, long last, int channel_mask
     int loops = 0 ;
     long ring_buffer_length ;
     double rb_left[BUFSIZE], rb_right[BUFSIZE] ;
-    int n_harmonics = 1 ;
 
     load_filter_preferences() ;
 
@@ -119,7 +124,18 @@ void filter_audio(struct sound_prefs *p, long first, long last, int channel_mask
     Fc = filter_prefs.Fc ;
     bandwidth = filter_prefs.bandwidth ;
 
-    g_print("type:%d Fc:%lg bandwidth:%lg\n", filter_type, Fc, bandwidth) ;
+    switch(filter_type) {
+	case LPF: g_print("LPF") ; break ;
+	case HPF: g_print("HPF") ; break ;
+	case BPF: g_print("BPF") ; break ;
+	case NOTCH: g_print("NOTCH") ; break ;
+	case PEQ: g_print("PEQ") ; break ;
+	case LSH: g_print("LSH") ; break ;
+	case HSH: g_print("HSH") ; break ;
+	default: g_print("UNKNOWN! ") ; break ;
+    }
+
+    g_print(" Fc:%lg bandwidth:%lg srate:%d\n", Fc, bandwidth,p->rate) ;
 
 
 /*  filtered_sample = current_sample - ring_buffer[j];  */
@@ -136,12 +152,11 @@ extern biquad *BiQuad_new(int type, smp_type dbGain, /* gain of filter */
 #endif
 
 #ifdef BIQUAD
-    biquad *iir_left[5], *iir_right[5] ;
+    biquad *iir_left, *iir_right ;
+
+    iir_left  = BiQuad_new(filter_type, dbGain, Fc, p->rate, bandwidth) ;
+    iir_right = BiQuad_new(filter_type, dbGain, Fc, p->rate, bandwidth) ;
 
-    for(f = 0 ; f < n_harmonics ; f++) {
-	iir_left[f]  = BiQuad_new(filter_type, dbGain, Fc*(f+1.0), p->rate, bandwidth) ;
-	iir_right[f] = BiQuad_new(filter_type,  dbGain, Fc*(f+1.0), p->rate, bandwidth) ;
-    }
     ring_buffer_length = p->rate / Fc + 0.5 ;
     for(i = 0 ; i < ring_buffer_length ; i++) {
 	rb_left[i] = 0.0 ;
@@ -154,9 +169,30 @@ extern biquad *BiQuad_new(int type, smp_type dbGain, /* gain of filter */
     get_iir_lp_coefs(Fc, &iir_right, p->rate) ;
 #endif
 
+    /* testing for filter response */
+    {
+	double freq, d_freq = 10 ;
+
+	g_print("left a0:%lg\n", iir_left->a0) ;
+	g_print("left a1:%lg\n", iir_left->a1) ;
+	g_print("left a2:%lg\n", iir_left->a2) ;
+	g_print("left b1:%lg\n", iir_left->a3) ;
+	g_print("left b2:%lg\n", iir_left->a4) ;
+
+	for(freq = 10 ; freq < 20001 ; freq += d_freq) {
+	    double from_formula ;
+	    double rr = BiQuad_response(freq, p->rate, iir_right, &from_formula) ;
+	    g_print("freq:%5.0lf response(dB):%10.2lg formula(dB):%10.2lg\n", freq, rr, from_formula) ;
+	    if(freq > 90) d_freq = 100 ;
+	    if(freq > 900) d_freq = 1000 ;
+	    if(freq > 9000) d_freq = 2000 ;
+	}
+    }
+
     current = first ;
 
-    push_status_text("Amplifying audio") ;
+    push_status_text("Filtering audio") ;
+    g_print("Filtering audio %ld to %ld\n", first, last) ;
     update_status_bar(0.0,STATUS_UPDATE_INTERVAL,TRUE) ;
 
     {
@@ -198,15 +234,7 @@ extern biquad *BiQuad_new(int type, smp_type dbGain, /* gain of filter */
 		    long dry_left = left[i];
 
 #ifdef BIQUAD
-/*  		    filtered_sample = dry_left - rb_left[i_left];  */
-/*  		    rb_left[i_left] = rb_left[i_left] * 0.0 + dry_left * 1.0;  */
-/*  		    i_left++;  */
-/*  		    i_left %= ring_buffer_length;  */
-/*  		    dry_left = filtered_sample ;  */
-
-		    for(f = 0 ; f < n_harmonics ; f++) {
-			wet_left = 32768.*BiQuad(dry_left/32768., iir_left[f]) ;
-		    }
+		    wet_left = 32768.*BiQuad(dry_left/32768., iir_left) ;
 #else
 		    wet_left = iir_filter(dry_left, &iir_left) ;
 #endif
@@ -218,15 +246,7 @@ extern biquad *BiQuad_new(int type, smp_type dbGain, /* gain of filter */
 		    long dry_right = right[i] ;
 
 #ifdef BIQUAD
-/*  		    filtered_sample = dry_right - rb_right[i_right];  */
-/*  		    rb_right[i_right] = rb_right[i_right] * 0.0 + dry_right * 1.0;  */
-/*  		    i_right++;  */
-/*  		    i_right %= ring_buffer_length;  */
-/*  		    dry_right = filtered_sample ;  */
-
-		    for(f = 0 ; f < n_harmonics ; f++) {
-			wet_right = 32768.0*BiQuad(dry_right/32768., iir_left[f]) ;
-		    }
+		    wet_right = 32768.0*BiQuad(dry_right/32768., iir_right) ;
 #else
 		    wet_right = iir_filter(dry_right, &iir_right) ;
 #endif
@@ -243,7 +263,7 @@ extern biquad *BiQuad_new(int type, smp_type dbGain, /* gain of filter */
 	    if(last - current < 10) loops++ ;
 
 	    if(loops > 5) {
-		warning("inifinite loop in amplify_audio, programming error\n") ;
+		warning("inifinite loop in filter_audio, programming error\n") ;
 	    }
 	}
 
@@ -255,10 +275,8 @@ extern biquad *BiQuad_new(int type, smp_type dbGain, /* gain of filter */
     pop_status_text() ;
 
 #ifdef BIQUAD
-    for(f = 0 ; f < n_harmonics ; f++) {
-	free(iir_left[f]) ;
-	free(iir_right[f]) ;
-    }
+	free(iir_left) ;
+	free(iir_right) ;
 #else
     filter_free(&iir_left) ;
     filter_free(&iir_right) ;
@@ -273,28 +291,71 @@ void type_window_select(GtkWidget * clist, gint row, gint column,
     filter_type = row2filter(row) ;
 }
 
+static GtkWidget *dbGain_entry ;
+static GtkWidget *freq_entry ;
+static GtkWidget *bandwidth_entry ;
+static struct sound_prefs local_sound_prefs ;
+static long first_sample, last_sample ;
+
+void show_response(GtkWidget *w, gpointer gdata)
+{
+    biquad *iir_left ;
+    double freq, d_freq=10 ;
+
+    Fc = atof(gtk_entry_get_text((GtkEntry *)freq_entry)) ;
+    dbGain = atof(gtk_entry_get_text((GtkEntry *)dbGain_entry)) ;
+    bandwidth = atof(gtk_entry_get_text((GtkEntry *)bandwidth_entry)) ;
+    iir_left  = BiQuad_new(filter_type, dbGain, Fc, 44100, bandwidth) ;
+
+    for(freq = 10 ; freq < 20001 ; freq += d_freq) {
+	double from_formula ;
+	double rl = BiQuad_response(freq, 44100, iir_left, &from_formula) ;
+	g_print("freq:%5.0lf response(dB):%10.4lg formula(dB):%10.4lg\n", freq, rl, from_formula) ;
+	if(freq > 90) d_freq = 100 ;
+	if(freq > 900) d_freq = 1000 ;
+	if(freq > 9000) d_freq = 2000 ;
+    }
+    free(iir_left) ;
+
+    g_print("first_sample:%ld last_sample:%ld\n", first_sample, last_sample) ;
+
+
+    {
+	struct denoise_prefs p ;
+	p.n_noise_samples = 10 ;
+	p.FFT_SIZE = 8192 ;
+	print_noise_sample(&local_sound_prefs, &p, first_sample, last_sample) ;
+    }
+}
+
 
 int filter_dialog(struct sound_prefs current, struct view *v)
 {
     GtkWidget *dlg, *dialog_table ;
     GtkWidget *feather_entry ;
-    GtkWidget *dbGain_entry ;
-    GtkWidget *freq_entry ;
-    GtkWidget *bandwidth_entry ;
     int dclose = 0 ;
     int row = 0 ;
     int dres ;
 
     GtkWidget *type_window_list;
+    GtkWidget *show_response_button ;
 
     gchar *type_window_titles[] = { "Filter Type" };
-    gchar *type_window_parms[4][1] = { {"Low Pass"},
+    gchar *type_window_parms[7][1] = { {"Low Pass"},
     {"High Pass"},
     {"Notch"},
-    {"Band Pass"}
+    {"Band Pass"},
+    {"Peaking EQ"},
+    {"Low Shelf Filter"},
+    {"High Shelf Filter"},
     };
 
+    local_sound_prefs = current ;
+    first_sample = v->selected_first_sample ;
+    last_sample = v->selected_last_sample ;
+
     load_filter_preferences();
+    filter_type = filter_prefs.filter_type ;
 
     type_window_list = gtk_clist_new_with_titles(1, type_window_titles);
     gtk_clist_set_selection_mode(GTK_CLIST(type_window_list),
@@ -303,6 +364,9 @@ int filter_dialog(struct sound_prefs current, struct view *v)
     gtk_clist_append(GTK_CLIST(type_window_list), type_window_parms[1]);
     gtk_clist_append(GTK_CLIST(type_window_list), type_window_parms[2]);
     gtk_clist_append(GTK_CLIST(type_window_list), type_window_parms[3]);
+    gtk_clist_append(GTK_CLIST(type_window_list), type_window_parms[4]);
+    gtk_clist_append(GTK_CLIST(type_window_list), type_window_parms[5]);
+    gtk_clist_append(GTK_CLIST(type_window_list), type_window_parms[6]);
 
     gtk_clist_select_row(GTK_CLIST(type_window_list),
 			 filter2row(filter_prefs.filter_type), 0);
@@ -324,13 +388,19 @@ int filter_dialog(struct sound_prefs current, struct view *v)
 
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), type_window_list,
 		       TRUE, TRUE, 0);
-    feather_entry = add_number_entry_with_label_int(feather_width, "Feather Width", dialog_table, row++) ;
-    dbGain_entry = add_number_entry_with_label_double(dbGain, "Gain (db)", dialog_table, row++) ;
-    freq_entry = add_number_entry_with_label_double(Fc, "Center frequency freq (hertz)", dialog_table, row++) ;
-    bandwidth_entry = add_number_entry_with_label_double(bandwidth, "bandwidth (octaves)", dialog_table, row++) ;
+    feather_entry = add_number_entry_with_label_int(filter_prefs.feather_width, "Feather Width", dialog_table, row++) ;
+    dbGain_entry = add_number_entry_with_label_double(filter_prefs.dbGain, "Gain (db)", dialog_table, row++) ;
+    freq_entry = add_number_entry_with_label_double(filter_prefs.Fc, "Center frequency freq (hertz)", dialog_table, row++) ;
+    bandwidth_entry = add_number_entry_with_label_double(filter_prefs.bandwidth, "bandwidth (octaves)", dialog_table, row++) ;
 
     gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dlg)->vbox), dialog_table, TRUE, TRUE, 0);
 
+    show_response_button = gtk_button_new_with_label("Show Response") ;
+    gtk_signal_connect(GTK_OBJECT(show_response_button), "clicked", GTK_SIGNAL_FUNC(show_response), NULL) ;
+    gtk_widget_show(show_response_button) ;
+
+    gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dlg)->vbox), show_response_button, TRUE, TRUE, 0);
+
     dres = gwc_dialog_run(GTK_DIALOG(dlg)) ;
 
     if(dres == 0) {
@@ -480,6 +550,15 @@ smp_type srate, smp_type bandwidth)
         return NULL;
     }
 
+
+    /* the cannonical coefficients */
+    b->can_a0 = a0 ;
+    b->can_a1 = a1 ;
+    b->can_a2 = a2 ;
+    b->can_b0 = b0 ;
+    b->can_b1 = b1 ;
+    b->can_b2 = b2 ;
+
     /* precompute the coefficients */
     b->a0 = b0 /a0;
     b->a1 = b1 /a0;
@@ -487,9 +566,71 @@ smp_type srate, smp_type bandwidth)
     b->a3 = a1 /a0;
     b->a4 = a2 /a0;
 
+#ifdef HARDWIRE
+    /* hardwire BPF at 10 Khz, srate = 44.1 Khz */
+    b->a0 =  1.0 ;
+    b->a1 =  0.0 ;
+    b->a2 = -1.0 ;
+    b->a3 = 0.1 ;
+    b->a4 = 0.9 ;
+
+    b->can_a0 = b->a0 ;
+    b->can_a1 = b->a1 ;
+    b->can_a2 = b->a2 ;
+    b->can_b0 = 1.0 ;
+    b->can_b1 = b->a3 ;
+    b->can_b2 = b->a4 ;
+#endif
+
     /* zero initial samples */
     b->x1 = b->x2 = 0;
     b->y1 = b->y2 = 0;
 
     return b;
 }
+
+#define M_SQR(x) ((x)*(x))
+
+double BiQuad_response(double freq, double srate, biquad *p, double *from_formula)
+{
+    double omega = 2.0 * M_PI * freq /(double)srate;
+    double phi = sin(omega/2) ;
+    double a0 = p->can_a0 ;
+    double a1 = p->can_a1 ;
+    double a2 = p->can_a2 ;
+    double b0 = p->can_b0 ;
+    double b1 = p->can_b1 ;
+    double b2 = p->can_b2 ;
+
+    double phi2 = phi*phi ;
+
+    *from_formula = 
+     10*log10( M_SQR(b0+b1+b2) - 4.0*(b0*b1 + 4.0*b0*b2 + b1*b2)*phi + 16.0*b0*b2*phi2 )
+     -10*log10( M_SQR(a0+a1+a2) - 4.0*(a0*a1 + 4.0*a0*a2 + a1*a2)*phi + 16.0*a0*a2*phi2 )  ;
+
+    int i ;
+    double sum_dry2 = 0, sum_wet2  = 0;
+
+    p->x1 = p->x2 = 0;
+    p->y1 = p->y2 = 0;
+
+    for(i = 0 ; i < srate ; i++) {
+	double x = sin(2.0*M_PI*freq/srate*(double)i) ;
+	double y ;
+	sum_dry2 += x*x ;
+	y = BiQuad(x, p) ;
+	sum_wet2 += y*y ;
+    }
+
+    return 20.0*log10(sum_wet2/sum_dry2) ;
+}
+
+
+/* from robert bristow-johnson's response, march 1, 2005
+http://groups.google.com/group/comp.dsp/browse_frm/thread/8c0fa8d396aeb444/a1bc5b63ac56b686
+
+20*log10[|H(e^jw)|] =
+
+ 10*log10[ (b0+b1+b2)^2 - 4*(b0*b1 + 4*b0*b2 + b1*b2)*phi + 16*b0*b2*phi^2 ]
+ -10*log10[ (a0+a1+a2)^2 - 4*(a0*a1 + 4*a0*a2 + a1*a2)*phi + 16*a0*a2*phi^2 ] 
+*/
