@@ -27,6 +27,14 @@
 #include "stat.h"
 #undef warning
 
+#define MESCHACH
+#ifndef MESCHACH
+#include <gsl/gsl_matrix.h>
+#include <gsl/gsl_vector.h>
+#include <gsl/gsl_permutation.h>
+#include <gsl/gsl_linalg.h>
+#endif
+
 void fit_cubic(fftw_real data[], int n, fftw_real estimated[]) ;
 
 #define FFT_MAX 8192
@@ -148,8 +156,97 @@ void fit_trig_basis(fftw_real data[], int n, fftw_real estimated[], int click_st
     }
 }
 
+
+#ifndef MESCHACH
+gsl_matrix *gsl_transp(gsl_matrix *m)
+{
+    int i,j,rows,cols ;
+    gsl_matrix *t ;
+
+    rows = m->size1 ;
+    cols = m->size2 ;
+
+    t = gsl_matrix_alloc(cols,rows) ;
+
+    for(i = 0 ; i < rows ; i++)
+	for(j = 0 ; j < cols ; j++)
+	    gsl_matrix_set(t, j, i, gsl_matrix_get(m, i, j)) ;
+
+    return t ;
+}
+
+gsl_vector *gsl_mv_mlt(gsl_matrix *m, gsl_vector *v)
+{
+    gsl_vector *r ;
+    int i,j,rows,cols ;
+
+    rows = m->size1 ;
+    cols = m->size2 ;
+
+    r = gsl_vector_alloc(cols) ;
+
+    for(j = 0 ; j < cols ; j++) {
+	double x = gsl_matrix_get(m, 0, j)*gsl_vector_get(v,0) ;
+	for(i = 1 ; i < rows ; i++)
+	    x += gsl_matrix_get(m, i, j)*gsl_vector_get(v,i) ;
+	gsl_vector_set(r, i, x) ;
+    }
+
+    return r ;
+}
+
+gsl_matrix  *gsl_m_mlt(gsl_matrix *m1, gsl_matrix *m2)
+{
+    gsl_matrix *r ;
+
+    int i,j,k,rows,cols,out_cols ;
+
+    rows = m1->size1 ;
+    cols = m1->size2 ;
+
+    out_cols = m2->size2 ;
+
+    r = gsl_matrix_alloc(rows,out_cols) ;
+
+    for(i = 0 ; i < rows ; i++) {
+	for(k = 0 ; k < out_cols ; k++) {
+	    double x = gsl_matrix_get(m1, i, 0)*gsl_matrix_get(m2,0,k) ;
+
+	    for(j = 1 ; j < cols ; j++)
+		x += gsl_matrix_get(m1, i, j)*gsl_matrix_get(m2,j,k) ;
+
+	    gsl_matrix_set(r, i, k, x) ;
+	}
+    }
+
+    return r ;
+}
+
+gsl_matrix * gsl_m_inverse(gsl_matrix *m)
+{
+    gsl_matrix *inverse,*ludecomp ;
+    gsl_permutation *perm ;
+    int s ;
+
+    inverse = gsl_matrix_alloc(m->size1,m->size2) ;
+    ludecomp = gsl_matrix_alloc(m->size1,m->size2) ;
+    perm = gsl_permutation_alloc(m->size1) ;
+
+    gsl_matrix_memcpy(ludecomp,m) ;
+    gsl_linalg_LU_decomp(ludecomp,perm,&s) ;
+    gsl_linalg_LU_invert(ludecomp,perm,inverse) ;
+
+    gsl_matrix_free(ludecomp) ;
+    gsl_permutation_free(perm) ;
+
+    return inverse ;
+}
+#endif
+
+
 int lsar_sample_restore(fftw_real data[], int firstbad, int lastbad, int siglen)
 {
+#ifdef MESCHACH
     int n_bad = lastbad - firstbad + 1 ;
     int autolen = 60 ;
     int i, j, rows, cols ;
@@ -264,6 +361,121 @@ int lsar_sample_restore(fftw_real data[], int firstbad, int lastbad, int siglen)
     V_FREE(rhs) ;
 
     return rcode ;
+#else
+    int n_bad = lastbad - firstbad + 1 ;
+    int autolen = 60 ;
+    int i, j, rows, cols ;
+    int rcode ;
+    gboolean clipped ;
+    double x[100], auto_coefs[101] ;
+    static gsl_matrix *A, *Au, *Aut, *AutmAu, *iAutmAu, *final ;
+    static gsl_vector *rhs, *sig, *sig_final ;
+
+    //estimate_region(data, firstbad, lastbad, siglen) ;
+    //return REPAIR_SUCCESS ;
+
+    autolen = (siglen-n_bad)/4 ;
+    //autolen *= 2 ;
+
+    if(autolen < 0) {
+	d_print("Autolen < 0!\n") ;
+	return REPAIR_FAILURE;
+    }
+
+    if(autolen > 3*n_bad) autolen = 3*n_bad ;
+    if(autolen > 100) autolen = 100 ;
+    //g_print("siglen:%d n_bad:%d Autolen:%d\n",siglen,n_bad,autolen) ;
+
+    sig = gsl_vector_alloc(siglen) ;
+    A = gsl_matrix_alloc(siglen-autolen, siglen) ;
+    Au = gsl_matrix_alloc(siglen-autolen, n_bad) ;
+
+    for(i = 0 ; i < siglen ; i++)
+	gsl_vector_set(sig,i, data[i]) ;
+
+    init_reg(autolen) ;
+
+    for(i = autolen ; i < firstbad ; i++) {
+	for(j = 0 ; j < autolen ; j++)
+	    x[j] = data[i - autolen + j] ;
+	sum_reg(x, data[i]) ;
+    }
+
+    for(i = lastbad+autolen+1 ; i < siglen ; i++) {
+	for(j = 0 ; j < autolen ; j++)
+	    x[j] = data[i - autolen + j] ;
+	sum_reg(x, data[i]) ;
+    }
+
+    if(estimate_reg(auto_coefs) == 1) {
+	rcode = SINGULAR_MATRIX ;
+    } else {
+
+	for(i = firstbad ; i <= lastbad ; i++) gsl_vector_set(sig,i,0.0) ;
+
+	rows = siglen - autolen ;
+	cols = siglen ;
+
+	for(i = 0 ; i < rows ; i++) {
+
+	    for(j = 0 ; j < autolen ; j++)
+		gsl_matrix_set(A,i,i+j,-auto_coefs[j]) ;
+
+	    gsl_matrix_set(A,i,i+autolen, 1.) ;
+
+	    for(j = firstbad ; j <= lastbad ; j++)
+		gsl_matrix_set(A,i,j-firstbad, gsl_matrix_get(A,i,j)) ;
+	}
+
+	for(j = firstbad ; j <= lastbad ; j++)
+	    gsl_vector_set(sig,j, 0.0) ;
+
+	Aut = gsl_transp(Au) ;
+
+	rhs = gsl_mv_mlt(A,sig) ;
+
+	AutmAu = gsl_m_mlt(Aut,Au) ;
+
+	iAutmAu = gsl_m_inverse(AutmAu) ;
+
+	final = gsl_m_mlt(iAutmAu,Aut) ;
+
+	sig_final = gsl_mv_mlt(final,rhs) ;
+
+	clipped = FALSE ;
+
+	for(j = firstbad ; j <= lastbad ; j++) {
+	    double tmp = -gsl_vector_get(sig_final,j-firstbad) ;
+	    if(tmp > 1.0) clipped = TRUE ;
+	    if(tmp < -1.0) clipped = TRUE ;
+	}
+
+	if(clipped == FALSE) {
+	    for(j = firstbad ; j <= lastbad ; j++) {
+		double tmp = -gsl_vector_get(sig_final,j-firstbad) ;
+		if(data[j] > 1.0) data[j] = 1.0 ;
+		if(data[j] < -1.0) data[j] = -1.0 ;
+	    }
+	}
+
+	if(clipped == FALSE)
+	    rcode =  REPAIR_SUCCESS ;
+	else
+	    rcode =  REPAIR_CLIPPED ;
+    }
+
+    gsl_matrix_free(A) ;
+    gsl_matrix_free(Au) ;
+    gsl_matrix_free(Aut) ;
+    gsl_matrix_free(AutmAu) ;
+    gsl_matrix_free(iAutmAu) ;
+    gsl_matrix_free(final) ;
+    gsl_vector_free(sig) ;
+    gsl_vector_free(sig_final) ;
+    gsl_vector_free(rhs) ;
+
+    return rcode ;
+#endif
 
 }
     
@@ -414,7 +626,9 @@ void get_hpf (long sample, fftw_real channel_data[], double *hpf, double *hpf_av
 		hpfl[posl] = data[-1] - 2. * data[0] + data[1];
 		hpfl[posl] *= hpfl[posl];
 		suml += hpfl[posl];
-		*hpf = sqrt(suml/(HPF_AVE_WIDTH_LOCAL-2));
+
+		/* bugfix -- thanks Paul Sanders 1/12/2007 */
+		*hpf = (suml > 0.0) ? sqrt(suml/(HPF_AVE_WIDTH_LOCAL-2)) : 0.0 ;
 	
 		sumb -= hpfb[posb];
 		hpfb[posb] = *hpf;
@@ -456,19 +670,19 @@ void get_hpf (long sample, fftw_real channel_data[], double *hpf, double *hpf_av
 /* bj 10/2002 end, but several changes in do_declick also */
 
 char *do_declick(struct sound_prefs *p, long first_sample, long last_sample, int channel_mask, double sensitivity, int repair,
-struct click_data *clicks, int iterate_flag)
+struct click_data *clicks, int iterate_flag, int leave_click_marks)
 {
     extern int declick_detector_type ;
 
     if(declick_detector_type == FFT_DETECT)
-	return do_declick_fft(p,first_sample,last_sample,channel_mask,sensitivity,repair,clicks,iterate_flag) ;
+	return do_declick_fft(p,first_sample,last_sample,channel_mask,sensitivity,repair,clicks,iterate_flag,leave_click_marks) ;
     else
-	return do_declick_hpf(p,first_sample,last_sample,channel_mask,sensitivity,repair,clicks,iterate_flag) ;
+	return do_declick_hpf(p,first_sample,last_sample,channel_mask,sensitivity,repair,clicks,iterate_flag,leave_click_marks) ;
 }
 
 #define DECLICK_MAX_FFT 128
 char *do_declick_fft(struct sound_prefs *p, long first_sample, long last_sample, int channel_mask, double sensitivity, int repair,
-struct click_data *clicks, int iterate_flag)
+struct click_data *clicks, int iterate_flag, int leave_click_marks)
 {
     static char results_buf[200] ;
     long window_first ;
@@ -503,7 +717,9 @@ struct click_data *clicks, int iterate_flag)
     }
 
     start_timer();
-    clicks->n_clicks = 0 ;
+
+    if(repair == FALSE || leave_click_marks == FALSE)
+	clicks->n_clicks = 0 ;
 
     n_repaired[0] = n_repaired[1] = 0 ;
     n_not_repaired[0] = n_not_repaired[1] = 0 ;
@@ -596,7 +812,7 @@ struct click_data *clicks, int iterate_flag)
 	    double mean_level[FFT_SIZE] ;
 	    double offset[FFT_WINDOW*2+1] ;
 	    double hgt_sum = 0 ;
-	    double mean, stderr, var, cv, stddev ;
+	    double mean, std_err, var, cv, stddev ;
 	    long click_start = 0 ;
 
 	    if(! ((channel+1) & channel_mask) ) continue ;
@@ -625,7 +841,7 @@ struct click_data *clicks, int iterate_flag)
 /*  		if(offset[i] < 0.0) offset[i] = 0.0 ;  */
 /*  	    }  */
 
-	    stats(offset, window_size, &mean, &stderr, &var, &cv, &stddev) ;
+	    stats(offset, window_size, &mean, &std_err, &var, &cv, &stddev) ;
 
 	    in_click = 0 ;
 
@@ -735,7 +951,7 @@ struct click_data *clicks, int iterate_flag)
 }
 
 char *do_declick_hpf(struct sound_prefs *p, long first_sample, long last_sample, int channel_mask, double sensitivity, int repair,
-struct click_data *clicks, int iterate_flag)
+struct click_data *clicks, int iterate_flag, int leave_click_marks)
 {
     static char results_buf[200] ;
     long min_sample, max_sample ;
@@ -765,7 +981,9 @@ struct click_data *clicks, int iterate_flag)
     }
 
     start_timer();
-    clicks->n_clicks = 0 ;
+
+    if(repair == FALSE || leave_click_marks == FALSE)
+	clicks->n_clicks = 0 ;
 
     n_repaired[0] = n_repaired[1] = 0 ;
     n_not_repaired[0] = n_not_repaired[1] = 0 ;
@@ -839,13 +1057,21 @@ struct click_data *clicks, int iterate_flag)
 		    get_hpf(i,pdata[channel],&hpf,&hpf_ave,&hpf_dev); 
 
 		    if(i <= sample - 2*(HPF_AVE_WING_BASE+HPF_AVE_WING_LOCAL)) {
-			if(in_click == 0 && hpf > 2.0*hpf_ave/sensitivity) {
+			int sample_is_in_click = 0 ;
+			if(hpf > 2.0*hpf_ave/sensitivity) sample_is_in_click = 1 ;
+#define test_clicks_as_dB 1
+			if(test_clicks_as_dB) {
+			    sample_is_in_click = 0 ;
+			    if(10.0*log10(hpf/hpf_ave) > sensitivity) sample_is_in_click = 1 ;
+			}
+
+			if(in_click == 0 && sample_is_in_click) {
 			    get_hpf(-HPF_DELTA_WIDTH,pdata[channel],&hpf,&hpf_ave,&hpf_dev);
 			    if (hpf > 2. * hpf_dev/sensitivity + hpf_ave) {
 				in_click = 1 ;
 				click_end = window_first + i ;
 			    }
-			} else if(in_click == 1 && hpf < 2.0*hpf_ave/sensitivity) {
+			} else if(in_click == 1 && !sample_is_in_click) {
 			    long width ;
 			    int result = DETECT_ONLY ;
 			    click_start = window_first+i ;
