@@ -75,6 +75,7 @@ double window_coef[DENOISE_MAX_FFT] ;
 double sum_window_wgts[DENOISE_MAX_FFT] ;
 double jg_upper[DENOISE_MAX_FFT][11] ;
 double jg_lower[DENOISE_MAX_FFT][11] ;
+/*  double *two_way_probs[DENOISE_MAX_FFT] ;  */
 
 void compute_bark_z(int FFT_SIZE, int rate)
 {
@@ -126,6 +127,10 @@ int get_window_delta(struct denoise_prefs *pDnprefs)
 
     if(pDnprefs->window_type == DENOISE_WINDOW_HANNING_OVERLAP_ADD) {
 	return pDnprefs->FFT_SIZE/2 ;
+#ifdef DENOISE_TRY_ONE_SAMPLE
+    } else if(pDnprefs->window_type == DENOISE_WINDOW_ONE_SAMPLE) {
+	return 1 ;
+#endif
     } else {
 	if(pDnprefs->window_type == DENOISE_WINDOW_BLACKMAN)
 	    return pDnprefs->FFT_SIZE/pDnprefs->smoothness ;
@@ -265,6 +270,10 @@ double fft_window(int k, int N, int window_type)
 	return blackman(k, N) ;
     } else if(window_type == DENOISE_WINDOW_BLACKMAN_HYBRID) {
 	return blackman_hybrid(k, N-N/4, N) ;
+#ifdef DENOISE_TRY_ONE_SAMPLE
+    } else if(window_type == DENOISE_WINDOW_ONE_SAMPLE) {
+	return hanning(k, N) ;
+#endif
     } else if(window_type == DENOISE_WINDOW_HANNING_OVERLAP_ADD) {
 	return hanning(k, N) ;
     }
@@ -272,6 +281,44 @@ double fft_window(int k, int N, int window_type)
     return 0.0 ;
 }
 
+double db2w(double db)
+{
+    return pow(10,db/10) ;
+}
+
+double ceramic_mic_response(double f)
+{
+    double db = 0.0 ;
+    if(f < 4000.0) {
+	db =  0.0 ;
+    } else if(f < 7000.0) {
+	db =  (f-4000.0)/7000.0*2.0 ;
+    } else if(f < 20000.0) {
+	db =  2.0 - ((f-7000.0)/13000.0)*12.0 ;
+    } else {
+	db =  -10.0 ;
+    }
+
+    return db2w(db) ;
+}
+
+void cdivide(double *a, double *b, double c, double d)
+{
+    double denom =  (c*c + d*d) ;
+    double anew, bnew ;
+
+    if(denom < 1.e-60) {
+	*a = 0.0 ;
+	*b = 0.0 ;
+	return ;
+    }
+
+    anew = (*a*c + *b*d) / denom ;
+    bnew = (*b*c - *a*d) / denom ;
+}
+
+#define bin2freq(r,s,k) ((double)r / 2.0 /(double)(s/2)*(double)k)
+
 int prev_sample[2] ;
 
 static fftw_real windowed[DENOISE_MAX_FFT] ;
@@ -469,77 +516,164 @@ void fft_remove_noise(fftw_real sample[], fftw_real noise_min2[], fftw_real nois
     if(0) {
 #endif
     } else {
-	for (k = 1; k <= pPrefs->FFT_SIZE/2 ; ++k) {
-	    if(noise2[k] > DBL_MIN) {
-		double gain, Fk, Gk ;
+	if(pPrefs->noise_suppression_method == DENOISE_EXPERIMENTAL) {
+	    /* this whole section is starting to play around with audio *restoration*, i.e. trying
+	     * to guess what was missing, and adding it back in.  Think of very old recordings
+	     * with that have a damped high frequency response */
+	    if(0) {
+		static int first=1 ;
+		/* try simple deconvolution */
+		for (k = 0 ; k <= pPrefs->FFT_SIZE/2 ; ++k) {
+		    /* Gk needs to look like the original response filter */
+		    double Gk = hanning(k,pPrefs->FFT_SIZE/2) ;
+		    double freq = bin2freq(pPrefs->rate,pPrefs->FFT_SIZE,k) ;
+		    Gk =  ceramic_mic_response(bin2freq(pPrefs->rate,pPrefs->FFT_SIZE,k)) ;
+		    Gk = Gk * Gk ;
 
-		if(pPrefs->noise_suppression_method == DENOISE_EM) {
-		    double Rpost = MAX(Y2[k]/noise2[k]-1.0, 0.0) ;
-		    double alpha = pPrefs->dn_gamma ;
-		    double Rprio ;
+		    if( !(k % 100) && first ) printf("k:%d freq:%0.1f Gk:%f\n", k, freq, Gk) ;
 
-		    if(prev_sample[ch] == 1)
-			Rprio = (1.0-alpha)*Rpost+alpha*gain_prev[k]*gain_prev[k]*Y2_prev[k]/noise2[k] ;
-		    else
-			Rprio = Rpost ;
-
-		    gain = gain_em(Rprio, Rpost, alpha) ;
-    /*  		g_print("Rpost:%lg Rprio:%lg gain:%lg gain_prev:%lg y2_prev:%lg\n", Rpost, Rprio, gain, gain_prev[k], Y2_prev[k]) ;  */
-		    gain_prev[k] = gain ;
-		    Y2_prev[k] = Y2[k] ;
-		} else if(pPrefs->noise_suppression_method == DENOISE_LORBER) {
-		    double SNRlocal = MAX(masked[k]/noise2[k]-1.0, 0.0) ;
-		    double SNRfilt, SNRprio ;
-		    double alpha ;
-
-		    if(prev_sample[ch] == 1) {
-			double eta = (1.0 + pPrefs->dn_gamma) / 2.0 ;  /* eta seems to be better closer to 1.0 than gamma */
-			SNRfilt = (1.-pPrefs->dn_gamma)*SNRlocal + pPrefs->dn_gamma*(sig_prev[k]/noise2[k]) ;
-			SNRprio = SNRfilt ; /* note, could use another parameter, like pPrefs->dn_gamma, here to compute SNRprio */
-			SNRprio = (1.-eta)*SNRlocal + eta*(sig_prev[k]/noise2[k]) ;
-		    }else {
-			SNRfilt = SNRlocal ;
-			SNRprio = SNRfilt ;
-		    }
-		    alpha = alpha_lorber(SNRprio) ;
-		    gain = 1.0 - alpha/(SNRfilt+1) ;
-		    sig_prev[k] = MAX(Y2[k]*gain,0.0) ;
-		} else if(pPrefs->noise_suppression_method == DENOISE_WOLFE_GODSILL) {
-		    double Rpost = MAX(Y2[k]/noise2[k]-1.0, 0.0) ;
-		    double alpha = pPrefs->dn_gamma ;
-		    double Rprio ;
-
-		    if(prev_sample[ch] == 1)
-			Rprio = (1.0-alpha)*Rpost+alpha*gain_prev[k]*gain_prev[k]*Y2_prev[k]/noise2[k] ;
-		    else
-			Rprio = Rpost ;
-
-		    Y2_prev[k] = Y2[k] ;
-
-		    if(Y2[k] > masked[k]) {
-			gain = MAX(masked[k]/Y2[k], Rprio/(Rprio+1.0)) ;
+		    if(k < pPrefs->FFT_SIZE/2) {
+/*  			cdivide(&out[k],&out[pPrefs->FFT_SIZE-k],Gk,0) ;  */
+			out[pPrefs->FFT_SIZE-k] /= Gk ;
+			out[k] /= Gk ;
 		    } else {
-			gain = 1.0 ;
+			out[k] /= Gk ;
 		    }
 
-		    gain_prev[k] = gain ;
 
-		} else if(pPrefs->noise_suppression_method == DENOISE_WEINER)
-		    gain = gain_weiner(Y2[k], noise2[k]) ;
-		else
-		    gain = gain_power_subtraction(Y2[k], noise2[k]) ;
+		    out[k] *= Gk ;
+		}
+		first=0 ;
+	    } else {
+		// Step 1.  Compute the linear regression coefs(a,b) for the function ln(power2) = a*k + b ;
+		double sumx = 0.0 ;
+		double sumy = 0.0 ;
+		double sumx2 = 0.0 ;
+		double sumxy = 0.0 ;
+		double n = 0 ;
+		double a, b ;
+		int k_cut_off = 2*pPrefs->FFT_SIZE/8 ;
+		fftw_real *r = gain_k ;
 
-		Fk = pPrefs->amount*(1.0-gain) ;
+		for (k = 1 ; k <= pPrefs->FFT_SIZE/2 ; ++k) {
+		    r[k] = 1.0 ;
+		}
 
-		if(Fk < 0.0) Fk = 0.0 ;
-		if(Fk > 1.0) Fk = 1.0 ;
+		for (k = 1 ; k < k_cut_off ; ++k) {
+		    double y = log(Y2[k]) ;
+		    sumx += (double)k ;
+		    sumy += y ;
+		    sumxy += (double)k*y ;
+		    sumx2 += (double)k*(double)k ;
+		    n++ ;
+		}
 
-		Gk =  1.0 - Fk ;
+		a = (n*(sumxy)-sumx*sumy) / (n*sumx2 - sumx*sumx) ;
+		b = (sumy - a*sumx) / n ;
 
-		out[k] *= Gk ;
-		if(k < pPrefs->FFT_SIZE/2) out[pPrefs->FFT_SIZE-k] *= Gk ;
+		printf("a,b %f %f\n", a, b) ;
 
-		gain_k[k] = Gk ;
+		// Step 2. Compute r[k], amount of energy present for each frequency bin relative to predicted
+
+		for (k = 1 ; k < k_cut_off ; ++k) {
+		    r[k] = log(Y2[k]) / (a*(double)k+b) ;
+		}
+
+    #define MAX_HARMONIC 3
+		// Step 3.  Compute gain in each bin >= k_cutoff ;
+		double harmonic_factor[6] = {1.0,1.0,1.0,1.0,1.0,1.0/8.0} ;
+		int harmonic ;
+
+		for(harmonic = 1 ; harmonic < MAX_HARMONIC+1 ; harmonic++) {
+		    double harmonic_fraction = 1.0 - ((double)harmonic / (double)(harmonic+1)) ;
+		    for (k = k_cut_off ; k <= pPrefs->FFT_SIZE/2 ; ++k) {
+			/* compute j, the index of the fundamental freq of which k is the harmonic */
+			int j = (int)((double)k * harmonic_fraction + 0.5) ;
+			double ln_p2_hat = r[j]*(a * (double)k + b) ;
+			double p2_hat = harmonic_factor[harmonic]*exp(ln_p2_hat) ;
+			double Gk = sqrt((p2_hat+Y2[k]) / Y2[k]) ;
+
+			if (k == pPrefs->FFT_SIZE/2) printf("harmonic, hf, j, p2_hat, Gk[maxfft], ln_p2_hat %d %f %d %f %f %f\n",
+								harmonic, harmonic_fraction, j, p2_hat, Gk, ln_p2_hat) ;
+
+			out[k] *= Gk ;
+			if(k < pPrefs->FFT_SIZE/2) out[pPrefs->FFT_SIZE-k] *= Gk ;
+		    }
+		}
+	    }
+	} else {
+	    for (k = 1; k <= pPrefs->FFT_SIZE/2 ; ++k) {
+		if(noise2[k] > DBL_MIN) {
+		    double gain, Fk, Gk ;
+
+		    if(pPrefs->noise_suppression_method == DENOISE_EM) {
+			double Rpost = MAX(Y2[k]/noise2[k]-1.0, 0.0) ;
+			double alpha = pPrefs->dn_gamma ;
+			double Rprio ;
+
+			if(prev_sample[ch] == 1)
+			    Rprio = (1.0-alpha)*Rpost+alpha*gain_prev[k]*gain_prev[k]*Y2_prev[k]/noise2[k] ;
+			else
+			    Rprio = Rpost ;
+
+			gain = gain_em(Rprio, Rpost, alpha) ;
+	/*  		g_print("Rpost:%lg Rprio:%lg gain:%lg gain_prev:%lg y2_prev:%lg\n", Rpost, Rprio, gain, gain_prev[k], Y2_prev[k]) ;  */
+			gain_prev[k] = gain ;
+			Y2_prev[k] = Y2[k] ;
+		    } else if(pPrefs->noise_suppression_method == DENOISE_LORBER) {
+			double SNRlocal = MAX(masked[k]/noise2[k]-1.0, 0.0) ;
+			double SNRfilt, SNRprio ;
+			double alpha ;
+
+			if(prev_sample[ch] == 1) {
+			    double eta = (1.0 + pPrefs->dn_gamma) / 2.0 ;  /* eta seems to be better closer to 1.0 than gamma */
+			    SNRfilt = (1.-pPrefs->dn_gamma)*SNRlocal + pPrefs->dn_gamma*(sig_prev[k]/noise2[k]) ;
+			    SNRprio = SNRfilt ; /* note, could use another parameter, like pPrefs->dn_gamma, here to compute SNRprio */
+			    SNRprio = (1.-eta)*SNRlocal + eta*(sig_prev[k]/noise2[k]) ;
+			}else {
+			    SNRfilt = SNRlocal ;
+			    SNRprio = SNRfilt ;
+			}
+			alpha = alpha_lorber(SNRprio) ;
+			gain = 1.0 - alpha/(SNRfilt+1) ;
+			sig_prev[k] = MAX(Y2[k]*gain,0.0) ;
+		    } else if(pPrefs->noise_suppression_method == DENOISE_WOLFE_GODSILL) {
+			double Rpost = MAX(Y2[k]/noise2[k]-1.0, 0.0) ;
+			double alpha = pPrefs->dn_gamma ;
+			double Rprio ;
+
+			if(prev_sample[ch] == 1)
+			    Rprio = (1.0-alpha)*Rpost+alpha*gain_prev[k]*gain_prev[k]*Y2_prev[k]/noise2[k] ;
+			else
+			    Rprio = Rpost ;
+
+			Y2_prev[k] = Y2[k] ;
+
+			if(Y2[k] > masked[k]) {
+			    gain = MAX(masked[k]/Y2[k], Rprio/(Rprio+1.0)) ;
+			} else {
+			    gain = 1.0 ;
+			}
+
+			gain_prev[k] = gain ;
+
+		    } else if(pPrefs->noise_suppression_method == DENOISE_WEINER)
+			gain = gain_weiner(Y2[k], noise2[k]) ;
+		    else
+			gain = gain_power_subtraction(Y2[k], noise2[k]) ;
+
+		    Fk = pPrefs->amount*(1.0-gain) ;
+
+		    if(Fk < 0.0) Fk = 0.0 ;
+		    if(Fk > 1.0) Fk = 1.0 ;
+
+		    Gk =  1.0 - Fk ;
+
+		    out[k] *= Gk ;
+		    if(k < pPrefs->FFT_SIZE/2) out[pPrefs->FFT_SIZE-k] *= Gk ;
+
+		    gain_k[k] = Gk ;
+		}
 	    }
 	}
     }
@@ -600,6 +734,9 @@ void print_denoise(char *header, struct denoise_prefs *pDnprefs)
 	case DENOISE_WINDOW_BLACKMAN : g_print("Blackman\n") ; break ;
 	case DENOISE_WINDOW_BLACKMAN_HYBRID : g_print("Blackman-hybrid\n") ; break ;
 	case DENOISE_WINDOW_HANNING_OVERLAP_ADD : g_print("Hanning-overlap-add\n") ; break ;
+#ifdef DENOISE_TRY_ONE_SAMPLE
+	case DENOISE_WINDOW_ONE_SAMPLE : g_print("One Sample\n") ; break ;
+#endif
 	default : g_print("!!!!!!!!! UNKNOWN !!!!!!!!!!!!\n") ; break ;
     }
     g_print("Suppression method:") ;
@@ -679,6 +816,11 @@ int denoise(struct sound_prefs *pPrefs, struct denoise_prefs *pDnprefs, long noi
 	left[k] = 0.0 ;
 	right_prev_frame[k] = 0.0 ;
 	right[k] = 0.0 ;
+/*  	two_way_probs[k] = (double *)calloc(pDnprefs->FFT_SIZE,sizeof(double)) ;  */
+/*  	if(two_way_probs[k] == NULL) {  */
+/*  	    fprintf(stderr, "Failed to malloc two_way_probs, %d of %d\n", k+1, pDnprefs->FFT_SIZE) ;  */
+/*  	    exit(1) ;  */
+/*  	}  */
     }
 
 
@@ -687,6 +829,7 @@ int denoise(struct sound_prefs *pPrefs, struct denoise_prefs *pDnprefs, long noi
     get_noise_sample(pPrefs, pDnprefs, noise_start, noise_end,
 		    left_noise_min, left_noise_max, left_noise_avg,
 		    right_noise_min, right_noise_max, right_noise_avg) ;
+    pDnprefs->rate = pPrefs->rate ;
 
     audio_normalize(denoise_normalize) ;
 
@@ -761,7 +904,14 @@ int denoise(struct sound_prefs *pPrefs, struct denoise_prefs *pDnprefs, long noi
 		}
 
 		write_fft_real_wavefile_data(left, right, current, current+pDnprefs->FFT_SIZE/2-1) ;
-
+#ifdef DENOISE_TRY_ONE_SAMPLE
+	    } else if(pDnprefs->window_type == DENOISE_WINDOW_ONE_SAMPLE) {
+		if(current == first_sample) {
+		    write_fft_real_wavefile_data(left, right, current, current+pDnprefs->FFT_SIZE/2-1) ;
+		} else {
+		    write_fft_real_wavefile_data(left, right, current, current) ;
+		}
+#endif
 	    } else {
 		write_fft_real_wavefile_data(left, right, current, MIN(tmplast, last_sample)) ;
 	    }
@@ -781,6 +931,10 @@ int denoise(struct sound_prefs *pPrefs, struct denoise_prefs *pDnprefs, long noi
     rfftw_destroy_plan(pBak);
 #endif /* HAVE_FFTW3 */
 
+    for(k = 0 ; k < pDnprefs->FFT_SIZE ; k++) {
+/*  	free(two_way_probs[k]) ;  */
+    }
+
     audio_normalize(1) ;
 
     update_status_bar(0.0,STATUS_UPDATE_INTERVAL,TRUE) ;
@@ -801,6 +955,8 @@ int print_noise_sample(struct sound_prefs *pPrefs, struct denoise_prefs *pDnpref
 
     fftw_real left_noise_max[DENOISE_MAX_FFT], right_noise_max[DENOISE_MAX_FFT], left_noise_avg[DENOISE_MAX_FFT] ;
     fftw_real left_noise_min[DENOISE_MAX_FFT], right_noise_min[DENOISE_MAX_FFT], right_noise_avg[DENOISE_MAX_FFT] ;
+    extern int MAXSAMPLEVALUE ;
+    double max = MAXSAMPLEVALUE * MAXSAMPLEVALUE ;
 
     get_noise_sample(pPrefs, pDnprefs, noise_start, noise_end,
 		    left_noise_min, left_noise_max, left_noise_avg,
@@ -812,7 +968,9 @@ int print_noise_sample(struct sound_prefs *pPrefs, struct denoise_prefs *pDnpref
 
     for(k = 1 ; k <= pDnprefs->FFT_SIZE/2 ; k++) {
 	double freq = (double)pPrefs->rate / 2.0 /(double)(pDnprefs->FFT_SIZE/2)*(double)k ;
-	fprintf(fp, "%10lg %12.1lf %12.1lf\n", freq, left_noise_avg[k], right_noise_avg[k]) ;
+	double db_left = 20.0*log10(left_noise_avg[k]/(max/2.0)) ;
+	double db_right = 20.0*log10(right_noise_avg[k]/(max/2.0)) ;
+	fprintf(fp, "%10lgHz %12.1lfdB %12.1lfdB\n", freq, db_left, db_right) ;
     }
 
     fclose(fp) ;
@@ -824,7 +982,7 @@ void get_noise_sample(struct sound_prefs *pPrefs, struct denoise_prefs *pDnprefs
 		    fftw_real *left_noise_min, fftw_real *left_noise_max, fftw_real *left_noise_avg,
 		    fftw_real *right_noise_min, fftw_real *right_noise_max, fftw_real *right_noise_avg)
 {
-    int i, k ;
+    int i, j, k ;
 #ifdef HAVE_FFTW3
     FFTW(plan) pForLeft, pForRight ;
 #else /* HAVE_FFTW3 */
@@ -870,6 +1028,14 @@ void get_noise_sample(struct sound_prefs *pPrefs, struct denoise_prefs *pDnprefs
 	    right[k] *= window_coef[k] ;
 	}
 
+	if(0 && pDnprefs->noise_suppression_method == DENOISE_EXPERIMENTAL) {
+	    for(k = 1 ; k <= pDnprefs->FFT_SIZE/2 ; k++) {
+		for(j = 1 ; j <= pDnprefs->FFT_SIZE/2 ; j++) {
+/*  		    two_way_probs[j][k] = 0.0 ;  */
+		}
+	    }
+	}
+
 #ifdef HAVE_FFTW3
 	FFTW(execute)(pForLeft);
 #else /* HAVE_FFTW3 */
@@ -878,15 +1044,37 @@ void get_noise_sample(struct sound_prefs *pPrefs, struct denoise_prefs *pDnprefs
 
 	/* convert noise sample to power spectrum */
 	for(k = 1 ; k <= pDnprefs->FFT_SIZE/2 ; k++) {
+	    double p2 ;
 	    if(k < pDnprefs->FFT_SIZE/2) {
-		left_noise_min[k] = MIN(left_noise_min[k], tmp[k] * tmp[k] + tmp[pDnprefs->FFT_SIZE-k]*tmp[pDnprefs->FFT_SIZE-k]) ;
-		left_noise_max[k] = MAX(left_noise_max[k], tmp[k] * tmp[k] + tmp[pDnprefs->FFT_SIZE-k]*tmp[pDnprefs->FFT_SIZE-k]) ;
-		left_noise_avg[k] += tmp[k] * tmp[k] + tmp[pDnprefs->FFT_SIZE-k]*tmp[pDnprefs->FFT_SIZE-k] ;
+		p2 = tmp[k] * tmp[k] + tmp[pDnprefs->FFT_SIZE-k]*tmp[pDnprefs->FFT_SIZE-k] ;
 	    } else {
 		/* Nyquist Frequency */
-		left_noise_min[k] = MIN(left_noise_min[k], tmp[k] * tmp[k]) ;
-		left_noise_max[k] = MAX(left_noise_max[k], tmp[k] * tmp[k]) ;
-		left_noise_avg[k] += tmp[k] * tmp[k] ;
+		p2 = tmp[k] * tmp[k] ;
+	    }
+	    left_noise_min[k] = MIN(left_noise_min[k], p2) ;
+	    left_noise_max[k] = MAX(left_noise_max[k], p2) ;
+	    left_noise_avg[k] += p2 ;
+	}
+
+	if(0 && pDnprefs->noise_suppression_method == DENOISE_EXPERIMENTAL) {
+	    for(k = 1 ; k <= pDnprefs->FFT_SIZE/2 ; k++) {
+		double p2 ;
+		if(k < pDnprefs->FFT_SIZE/2) {
+		    p2 = tmp[k] * tmp[k] + tmp[pDnprefs->FFT_SIZE-k]*tmp[pDnprefs->FFT_SIZE-k] ;
+		} else {
+		    /* Nyquist Frequency */
+		    p2 = tmp[k] * tmp[k] ;
+		}
+		for(j = k+k/2 ; j <= pDnprefs->FFT_SIZE/2 ; j++) {
+		    double p2j ;
+		    if(j < pDnprefs->FFT_SIZE/2) {
+			p2j = tmp[j] * tmp[j] + tmp[pDnprefs->FFT_SIZE-j]*tmp[pDnprefs->FFT_SIZE-j] ;
+		    } else {
+			/* Nyquist Frequency */
+			p2j = tmp[j] * tmp[j] ;
+		    }
+/*  		    two_way_probs[j][k] = MAX(two_way_probs[j][k],p2j/p2) ;  */
+		}
 	    }
 	}
 
@@ -898,15 +1086,37 @@ void get_noise_sample(struct sound_prefs *pPrefs, struct denoise_prefs *pDnprefs
 
 	/* convert noise sample to power spectrum */
 	for(k = 1 ; k <= pDnprefs->FFT_SIZE/2 ; k++) {
+	    double p2 ;
 	    if(k < pDnprefs->FFT_SIZE/2) {
-		right_noise_min[k] = MIN(right_noise_min[k], tmp[k] * tmp[k] + tmp[pDnprefs->FFT_SIZE-k]*tmp[pDnprefs->FFT_SIZE-k]) ;
-		right_noise_max[k] = MAX(right_noise_max[k], tmp[k] * tmp[k] + tmp[pDnprefs->FFT_SIZE-k]*tmp[pDnprefs->FFT_SIZE-k]) ;
-		right_noise_avg[k] += tmp[k] * tmp[k] + tmp[pDnprefs->FFT_SIZE-k]*tmp[pDnprefs->FFT_SIZE-k] ;
+		p2 = tmp[k] * tmp[k] + tmp[pDnprefs->FFT_SIZE-k]*tmp[pDnprefs->FFT_SIZE-k] ;
 	    } else {
 		/* Nyquist Frequency */
-		right_noise_min[k] = MIN(right_noise_min[k], tmp[k] * tmp[k]) ;
-		right_noise_max[k] = MAX(right_noise_max[k], tmp[k] * tmp[k]) ;
-		right_noise_avg[k] += tmp[k] * tmp[k] ;
+		p2 = tmp[k] * tmp[k] ;
+	    }
+	    right_noise_min[k] = MIN(right_noise_min[k], p2) ;
+	    right_noise_max[k] = MAX(right_noise_max[k], p2) ;
+	    right_noise_avg[k] += p2 ;
+	}
+
+	if(0 && pDnprefs->noise_suppression_method == DENOISE_EXPERIMENTAL) {
+	    for(k = 1 ; k <= pDnprefs->FFT_SIZE/2 ; k++) {
+		double p2 ;
+		if(k < pDnprefs->FFT_SIZE/2) {
+		    p2 = tmp[k] * tmp[k] + tmp[pDnprefs->FFT_SIZE-k]*tmp[pDnprefs->FFT_SIZE-k] ;
+		} else {
+		    /* Nyquist Frequency */
+		    p2 = tmp[k] * tmp[k] ;
+		}
+		for(j = k+k/2 ; j <= pDnprefs->FFT_SIZE/2 ; j++) {
+		    double p2j ;
+		    if(j < pDnprefs->FFT_SIZE/2) {
+			p2j = tmp[j] * tmp[j] + tmp[pDnprefs->FFT_SIZE-j]*tmp[pDnprefs->FFT_SIZE-j] ;
+		    } else {
+			/* Nyquist Frequency */
+			p2j = tmp[j] * tmp[j] ;
+		    }
+/*  		    two_way_probs[j][k] = MAX(two_way_probs[j][k],p2j/p2) ;  */
+		}
 	    }
 	}
 
