@@ -486,6 +486,7 @@ void draw_sonogram(struct view *v, struct sound_prefs *pPrefs, GtkWidget *da, do
     for(FFT_SIZE = 64 ; FFT_SIZE < FFT_MAX ; FFT_SIZE *= 2)
 	if(FFT_SIZE > samples_per_pixel*2)
 	    break ;
+/*      g_print("Sonogram FFT_SIZE:%d\n", FFT_SIZE) ;  */
 
     m = FFT_SIZE/4 ;
 
@@ -670,6 +671,40 @@ ly2_tbl[0] = 99999999;
        v->canvas_width, v->canvas_height); 
     gdk_image_destroy(image);
 
+#define DRAW_CLICKS_NOT
+#ifdef DRAW_CLICKS
+
+#define CLICK_WINDOW_SIZE 1000
+#define CLICK_WINDOW_STEP 800
+
+    int iwindow ;
+
+    /* this goes BACKWARDS through the data */
+    for(iwindow = v->last_sample; iwindow >= v->first_sample  ; iwindow -= CLICK_WINDOW_STEP) {
+	int ifirst = iwindow-CLICK_WINDOW_SIZE ;
+	if(ifirst < 0) ifirst=0 ;
+	int window_size = iwindow-ifirst ;
+	fftw_real pdata[2][CLICK_WINDOW_SIZE] ;
+	fprintf(stderr, "ifirst, iwindow, %d %d\n", ifirst, iwindow) ;
+	read_fft_real_wavefile_data(pdata[0],  pdata[1], ifirst, iwindow) ;
+	int j ;
+
+	for(j = 0 ; j < window_size+1 ; j++) {
+	    int i = iwindow-j ;
+	    for(channel = 0 ; channel < 2 ; channel++) {
+		double hpf, hpf_ave, hpf_dev;
+		double hpf2, hpf2_ave, hpf2_dev;
+
+		get_hpf(i,pdata[channel],&hpf,&hpf_ave,&hpf_dev,&hpf2,&hpf2_ave,&hpf2_dev); 
+		ly1 = AtoS(hpf,200.0,channel,v->canvas_height) ;
+		ly2 = AtoS(hpf2,200.0,channel,v->canvas_height) ;
+		int x = sample_to_pixel(v, i) ;
+		draw_a_line(da->style->white_gc, x, ly1, x, ly2, green_color, 0) ;
+	    }
+	}
+    }
+#endif /* DRAW_FFT_CLICKS */
+
 #ifdef DRAW_FFT_CLICKS
     for(channel = 0 ; channel < 2 ; channel++) {
 	double mean_level[MAXSH] ;
@@ -1124,10 +1159,64 @@ void redraw(struct view *v, struct sound_prefs *p, GtkWidget *da, int cursor_fla
 	}
 	draw_a_line(MyGC, 0, zero_left, v->canvas_width-1, zero_left, blue_color, 0) ;
 	draw_a_line(MyGC, 0, zero_right, v->canvas_width-1, zero_right, blue_color, 0) ;
+
+	int n_view_samples = v->last_sample-v->first_sample+1 ;
+
+	if(n_view_samples < 20000) {
+	    int x ;
+	    /* draw the RMSE of samples in a green line */
+	    double last_left_rmse = -1.0 ;
+	    double last_right_rmse = -1.0 ;
+	    double left_rmse = -1 ;
+	    double right_rmse = -1 ;
+	    double samp_width = 1./samples_per_pixel ;
+
+	    for(x = 0 ; x < v->canvas_width ; x++) {
+		int s_at_x = (double)x / (double)(v->canvas_width-1) * n_view_samples + v->first_sample ;
+		int first = MAX(s_at_x-50, 0) ;
+		int last = MIN(s_at_x+50, v->n_samples-1) ;
+		n = read_fft_real_wavefile_data(left, right, first, last) ;
+
+		left_rmse = 0 ; /* first use these for sums */
+		right_rmse = 0 ;
+		for(i = 0 ; i < n-1 ; i++) {
+		    left_rmse += left[i] * left[i] ;
+		    right_rmse += right[i] * right[i] ;
+		}
+
+		left_rmse = sqrt(left_rmse/((double)n+1.e-30)) ;
+		right_rmse = sqrt(right_rmse/((double)n+1.e-30)) ;
+
+		if(last_left_rmse > -1.0) {
+		    int x1, x2, ly1, ly2, ry1, ry2 ;
+
+		    ly1 = AtoS(-last_left_rmse,1.0,0,v->canvas_height) ;
+		    ly2 = AtoS(-left_rmse,1.0,0,v->canvas_height) ;
+		    ry1 = AtoS(-last_right_rmse,1.0,1,v->canvas_height) ;
+		    ry2 = AtoS(-right_rmse,1.0,1,v->canvas_height) ;
+
+		    x1 = x ;
+		    x2 = x1 + 1 ;
+
+		    draw_a_line(MyGC, x1, ly1, x2, ly2, green_color, draw_flag) ;
+		    draw_a_line(MyGC, x1, ry1, x2, ry2, green_color, draw_flag) ;
+		}
+		last_left_rmse = left_rmse ;
+		last_right_rmse = right_rmse ;
+	    }
+	}
+
     }
 
     draw_a_rect(MyGC, 0, y1, v->canvas_width-1, y1+8, black_color) ;
 
+    for(i = 0 ; i < n_markers ; i++) {
+	if(markers[i] >= v->first_sample && markers[i] <= v->last_sample) {
+	    int x = sample_to_pixel(v, markers[i]) ;
+	    draw_a_line(MyGC, x, 0, x, v->canvas_height-1, blue_color, 0) ;
+	}
+    }
+
     { 
 	for (i = 0; i < num_song_markers; i++) {
 	    if(song_markers[i] >= v->first_sample && song_markers[i] <= v->last_sample) {
@@ -1137,13 +1226,6 @@ void redraw(struct view *v, struct sound_prefs *p, GtkWidget *da, int cursor_fla
 	}
     }
 
-    for(i = 0 ; i < n_markers ; i++) {
-	if(markers[i] >= v->first_sample && markers[i] <= v->last_sample) {
-	    int x = sample_to_pixel(v, markers[i]) ;
-	    draw_a_line(MyGC, x, 0, x, v->canvas_height-1, blue_color, 0) ;
-	}
-    }
-
 /*      if(v->selection_region == TRUE) {  */
 	paint_screen_with_highlight(v, da, y1, y2, cursor_flag) ;
 /*      } else {  */
