@@ -38,26 +38,29 @@
 SNDFILE *in_fd;
 SNDFILE *out_fd;
 char *options[255];
+char pipe_name[256] ;
 
 SF_INFO insfinfo, outsfinfo;	/* sound file format descriptions for input and output */
 extern struct encoding_prefs encoding_prefs;
 
 struct stat filestat;
 
-int build_options(int fmt, char *newfilename)
+int build_options(int fmt, char *newfilename, char *trackname)
 {
     int i, optcnt = 0;
 
+    sprintf(pipe_name, ".gwc_pipe%d\n", (int)getpid()) ;
+
     /* remove any old options */
     for (i = 0; i < 255; i++)
-	options[i] = NULL;
+	options[i] = (char *)NULL;
 
     if (insfinfo.channels > 2) {
 	warning("We only support mono and stereo encodings");
 	return (1);
     }
 
-    if (fmt == OGGFMT) {
+    if (fmt == OGG_FMT) {
 
 	/* Check if defined encoder location and it exists */
 	if ((strlen(encoding_prefs.oggloc) == 0)
@@ -188,7 +191,7 @@ int build_options(int fmt, char *newfilename)
 	    return (1);
 	    break;		/* should not be here unknown value */
 	}
-    } else {
+    } else if(fmt == MP3_FMT) {
 	/* Check if defined encoder location and it exists */
 	if ((strlen(encoding_prefs.mp3loc) == 0)
 	    || (stat(encoding_prefs.mp3loc, &filestat) == -1)) {
@@ -442,6 +445,53 @@ int build_options(int fmt, char *newfilename)
 	/* All other options come before the input and output filenames */
 	options[optcnt] = "-";	/* Stdin */
 	options[optcnt + 1] = newfilename;
+    } else {
+	/* fmt == MP3_SIMPLE_FMT */
+	/* Check if defined encoder location and it exists */
+	if ((strlen(encoding_prefs.mp3loc) == 0)
+	    || (stat(encoding_prefs.mp3loc, &filestat) == -1)) {
+	    warning("Encoder for MP3 is not defined correctly\n");
+	    return 1;
+	}
+	/* Required Settings */
+
+	options[optcnt] = (char *) basename(encoding_prefs.mp3loc);	/* argv[0] is the executable */
+	optcnt++;
+	options[optcnt] = "-h";	/* use recommended quality setting from lame */
+	optcnt++;
+	options[optcnt] = "-r";	/* read raw */
+	optcnt++;
+	options[optcnt] = "--bitwidth";
+	optcnt++;
+	options[optcnt] = "16";
+	optcnt++;
+	options[optcnt] = "-V";
+	optcnt++;
+	options[optcnt] = encoding_prefs.mp3_quality_level ;
+	optcnt++;
+
+	if ((strlen(encoding_prefs.artist) > 0)) {
+	    options[optcnt] = "--ta";
+	    options[optcnt + 1] = encoding_prefs.artist;
+	    optcnt += 2;
+	}
+
+	if ((strlen(encoding_prefs.album) > 0)) {
+	    options[optcnt] = "--tl";
+	    options[optcnt + 1] = encoding_prefs.album;
+	    optcnt += 2;
+	}
+
+	if ((strlen(trackname) > 0)) {
+	    options[optcnt] = "--tt";
+	    options[optcnt + 1] = trackname ;
+	    optcnt += 2;
+	}
+
+	/* All other options come before the input and output filenames */
+	options[optcnt] = "-";	/* Stdin */
+	options[optcnt] = pipe_name ;	/* named pipe */
+	options[optcnt + 1] = newfilename;
     }
 
     return (0);
@@ -449,7 +499,7 @@ int build_options(int fmt, char *newfilename)
 
 
 int encode(int mode, char *origfilename, char *newfilename, long start,
-	   long length)
+	   long length, char *trackname)
 {
 
     int ret;
@@ -459,7 +509,7 @@ int encode(int mode, char *origfilename, char *newfilename, long start,
     if(fd_test != -1) {
 	close(fd_test) ;
 	if(yesno("File exists, overwrite?"))
-	    return ;
+	    return 0 ;
     }
 
     signal(SIGCHLD, SIG_IGN);	/* Make sure we dont create a zombie */
@@ -494,7 +544,7 @@ int encode(int mode, char *origfilename, char *newfilename, long start,
     }
 
 
-    ret = build_options(mode, newfilename);	/* create options based on encoder chosen */
+    ret = build_options(mode, newfilename, trackname);	/* create options based on encoder chosen */
 
     if (ret != 0) {
 	sf_close(in_fd);
@@ -503,7 +553,7 @@ int encode(int mode, char *origfilename, char *newfilename, long start,
     /* set format for output */
     outsfinfo.format = (SF_FORMAT_RAW | SF_FORMAT_PCM_16);
 
-    ret = start_encode(mode, newfilename, start, length);
+    ret = start_encode(mode, newfilename, start, length, origfilename);
 
     return ret;
 
@@ -515,12 +565,51 @@ gint encode_progress (gfloat pvalue) ;
 void destroy_progress_window(void) ;
 gint stop_encoding = FALSE ;
 
-int start_encode(int mode, char *newfilename, long start, long length)
+int start_encode_old(int mode, char *newfilename, long start, long length, char *origfilename)
 {
     long samples_read;
     long ctr;
     long numframes = 0;
-    int f_des[2], pid;
+    int f_des[2], child_pid;
+    int i=0 ;
+    char cmd[2048] ;
+
+    printf("Encoding using %s\n", encoding_prefs.mp3loc) ;
+    printf("   ") ;
+
+    for(i = 0 ; options[i] != (char *)NULL ; i++)
+	printf("\'%s\'", options[i]) ;
+
+    printf("\n") ;
+
+    sprintf(cmd, "sox %s -t raw - trim %ld\s %ld\s |", origfilename, start, length) ;
+
+    for(i = 0 ; options[i] != (char *)NULL ; i++) {
+	int j ;
+	strcat(cmd, " ") ;
+
+	for(j = 0 ; j < strlen(options[i]) ; j++) {
+	    char buf[10] ;
+	    int need_esc=0 ;
+
+	    if(options[i][j] == ' ') need_esc = 1 ;
+	    if(options[i][j] == '\'') need_esc = 1 ;
+	    if(options[i][j] == '\"') need_esc = 1 ;
+
+	    if(need_esc)
+		sprintf(buf, "\\%c", options[i][j]) ;
+	    else
+		sprintf(buf, "%c", options[i][j]) ;
+
+	    strcat(cmd, buf) ;
+	}
+    }
+
+    printf("CMD:\n'%s\'\n", cmd) ;
+    system(cmd) ;
+    return 0 ;
+
+
 
     /* Create a  pipe to send data through - must do first */
 
@@ -531,22 +620,28 @@ int start_encode(int mode, char *newfilename, long start, long length)
     }
     /* fork to allow us to create a child process for processing data */
 
-    if ((pid = fork()) == 0) {
+    if ((child_pid = fork()) == 0) {
 
 	/* Child (or proc 2) */
 
 	close(0);		/* redirecting stdin fd=0 */
 	dup(f_des[0]);		/* duplicate pipe read with stdin i.e. 0=3 */
 
-	close(f_des[1]);	/* child proc doe snot need stdout of pipe */
+	close(f_des[1]);	/* child proc does not need stdout of pipe */
 
-	if (mode == OGGFMT) {
+	if (mode == OGG_FMT) {
 	    /* execute ogg encoder using prebuilt options */
 	    if (execvp(encoding_prefs.oggloc, options) == -1) {
 		warning("Failed to process audio file\n");
 		return (1);
 	    }
-	} else {
+	} else if (mode == MP3_FMT) {
+	    /* execute mp3 encoder using prebuilt options */
+	    if (execvp(encoding_prefs.mp3loc, options) == -1) {
+		warning("Failed to process audio file\n");
+		return (1);
+	    }
+	} else if (mode == MP3_SIMPLE_FMT) {
 	    /* execute mp3 encoder using prebuilt options */
 	    if (execvp(encoding_prefs.mp3loc, options) == -1) {
 		warning("Failed to process audio file\n");
@@ -557,7 +652,9 @@ int start_encode(int mode, char *newfilename, long start, long length)
 
     } else {
 	double *framebuf = NULL ;
+	FILE *fp ;
 	/* Parent */
+	fprintf(stderr, "encoding child pid is %d\n", child_pid) ;
 	close(1);		/* redirecting stdout fd=1 */
 	dup(f_des[1]);		/* duplicate pipe write with stdout i.e. 1=4 */
 	close(f_des[0]);	/* closing stdin of pipe for this process not used */
@@ -602,8 +699,149 @@ int start_encode(int mode, char *newfilename, long start, long length)
 	close(f_des[0]);	/*  overall parent closes pipes so that  */
 	close(f_des[1]);	/*  processes terminate on time */
 
-	freopen("/dev/tty", "w", stdout);	/* reopen tty for writing on stdout */
-	/*  /dev/fd/1 or /dev/stdout is not having right perms */
+	fp = freopen("/dev/tty", "w", stdout);	/* reopen tty for writing on stdout */
+
+	if(fp == NULL) {
+	    fprintf(stderr, "FAILED to reopen stdout!\n") ;
+	} else {
+	    stdout = fp ;
+	    fprintf(stderr, "SUCCEEDED to reopen stdout!\n") ;
+	    printf("Finished encoding using %s\n", encoding_prefs.mp3loc) ;
+	}
+    }
+
+    return (0);
+
+}
+
+int start_encode(int mode, char *newfilename, long start, long length, char *origfilename)
+{
+    long samples_read;
+    long ctr;
+    long numframes = 0;
+    int f_des[2], child_pid;
+    int i=0 ;
+    int use_sox = 0 ;
+    char cmd[2048] ;
+    char *exec_loc ;
+
+    if (mode == OGG_FMT) {
+	/* execute ogg encoder using prebuilt options */
+	exec_loc = encoding_prefs.oggloc ;
+    } else if (mode == MP3_FMT) {
+	/* execute mp3 encoder using prebuilt options */
+	exec_loc = encoding_prefs.mp3loc ;
+    } else if (mode == MP3_SIMPLE_FMT) {
+	/* execute mp3 encoder using prebuilt options */
+	exec_loc = encoding_prefs.mp3loc ;
+    }
+
+    printf("Encoding using %s\n", exec_loc) ;
+    printf("   ") ;
+
+    for(i = 0 ; options[i] != (char *)NULL ; i++)
+	printf("\'%s\'", options[i]) ;
+
+    printf("\n") ;
+
+    if(use_sox) {
+	sprintf(cmd, "sox %s -t raw - trim %ld\s %ld\s |", origfilename, start, length) ;
+
+	for(i = 0 ; options[i] != (char *)NULL ; i++) {
+	    int j ;
+	    strcat(cmd, " ") ;
+
+	    for(j = 0 ; j < strlen(options[i]) ; j++) {
+		char buf[10] ;
+		int need_esc=0 ;
+
+		if(options[i][j] == ' ') need_esc = 1 ;
+		if(options[i][j] == '\'') need_esc = 1 ;
+		if(options[i][j] == '\"') need_esc = 1 ;
+
+		if(need_esc)
+		    sprintf(buf, "\\%c", options[i][j]) ;
+		else
+		    sprintf(buf, "%c", options[i][j]) ;
+
+		strcat(cmd, buf) ;
+	    }
+	}
+
+	printf("CMD:\n'%s\'\n", cmd) ;
+	system(cmd) ;
+	return 0 ;
+    }
+
+
+
+
+    /* Create a  pipe to send data through - must do first */
+    /* now using a named pipe */
+    if(mkfifo(pipe_name, S_IRWXU)) {
+	static char buf[254] ;
+	snprintf(buf,255,"Failed to open named pipe to transfer data to %s, cannot proceed", exec_loc) ;
+	warning(buf) ;
+	return 1 ;
+    }
+
+    /* fork to allow us to create a child process for processing data */
+
+    if ((child_pid = fork()) == 0) {
+
+	/* Child (or proc 2) */
+	if (execvp(exec_loc, options) == -1) {
+	    warning("Failed to process audio file\n");
+	    return (1);
+	}
+
+    } else {
+	double *framebuf = NULL ;
+	FILE *fp ;
+	/* Parent */
+	fprintf(stderr, "encoding child pid is %d\n", child_pid) ;
+
+	//close(1);		/* redirecting stdout fd=1 */
+	//dup(f_des[1]);		/* duplicate pipe write with stdout i.e. 1=4 */
+	//close(f_des[0]);	/* closing stdin of pipe for this process not used */
+
+	/*  open output to named_pipe */
+	out_fd = sf_open(pipe_name, SFM_WRITE, &outsfinfo);
+
+	if (out_fd == NULL) {
+	    warning("Failed to open output file\n");
+	    sf_close(in_fd);
+	    unlink(pipe_name) ;
+	    return (1);
+	}
+	/*  put out raw data */
+
+	/* Make sure frame buffer size is multiple of # channels */
+	numframes = 44100;
+	framebuf = (double*)calloc(numframes * insfinfo.channels, sizeof(double));
+	if (framebuf == NULL) {
+	    warning( "failed to allocate intermediate buffer\n");
+	    sf_close(in_fd);
+	    unlink(pipe_name) ;
+	    return (1);
+	}
+
+	create_progress_window() ;
+
+	/* read in blocks of size  buffer and  send out converted to new sound format */
+	for (ctr = 0; ctr < length; ctr += samples_read) {
+	    if(stop_encoding == TRUE) break ;
+	    samples_read = sf_readf_double(in_fd, framebuf, numframes);
+	    sf_writef_double(out_fd, framebuf, samples_read);
+	    encode_progress ((double)ctr/(double)length) ;
+	}
+
+	destroy_progress_window() ;
+
+	/*  Main  Parent (proc 1) */
+	sf_close(in_fd);	/* done reading/writing from program */
+	sf_close(out_fd);
+	unlink(pipe_name) ;
     }
 
     return (0);
