@@ -1,5 +1,6 @@
-#define OGGFMT 1000
-#define MP3FMT 2000
+#define OGG_FMT 1000
+#define MP3_FMT 2000
+#define MP3_SIMPLE_FMT 3000
 
 struct encoding_prefs {
 	/* MP3 */
@@ -23,6 +24,8 @@ struct encoding_prefs {
 	char mp3_lowpass_freq[6];
 	char mp3_highpass_freq[6];
 	char mp3loc[256]; /* Full path and executable for mp3 encoder */
+	char artist[256];
+	char album[256];
 
 	/* OGG Vorbis */
 	char ogg_quality_level[7]; /*  6 characters */
