@@ -528,6 +528,51 @@ int yesno(char *msg)
     return dres;
 }
 
+int prompt_user(char *msg, char *s, int maxlen)
+{
+    GtkWidget *dlg, *text, *entry ;
+    int dres;
+
+    dlg = gtk_dialog_new_with_buttons("Input Requested",
+    				        GTK_WINDOW(main_window),
+				        GTK_DIALOG_DESTROY_WITH_PARENT,
+					GTK_STOCK_OK,
+					GTK_RESPONSE_OK,
+					 GTK_STOCK_CANCEL,
+					 GTK_RESPONSE_CANCEL,
+					 NULL);
+
+    text = gtk_label_new(msg);
+    gtk_widget_show(text);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), text, TRUE, TRUE, 0);
+
+    entry = gtk_entry_new_with_max_length(maxlen);
+
+    if(strlen(s) > 0)
+	gtk_entry_set_text(GTK_ENTRY(entry), s);
+
+    gtk_widget_show(entry);
+
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), entry, TRUE, TRUE, 0);
+
+    gtk_widget_show_all(dlg) ;
+
+    dres = gtk_dialog_run(GTK_DIALOG(dlg));
+
+    if (dres == GTK_RESPONSE_NONE || dres == GTK_RESPONSE_NO) {
+	dres = 1 ;		/* return we clicked cancel */
+    } else {
+	dres = 0 ; 		/* return we clicked yes */
+	strcpy(s, gtk_entry_get_text(GTK_ENTRY(entry)));
+    }
+
+    gtk_widget_destroy(dlg) ;
+
+    main_redraw(FALSE, TRUE);
+
+    return dres;
+}
+
 void show_help(const char *filename)
 {
     GError *err = NULL ;
@@ -1392,6 +1437,13 @@ void split_audio_on_markers(GtkWidget * widget, gpointer data)
 	long first_sample = 0 ;
 	long last_sample = song_markers[0] ;
 
+/*  	for(i = 0 ; i < num_song_markers ; i++) {  */
+/*  	    printf("marker %2d at sample %d\n", i, song_markers[i]) ;  */
+/*  	}  */
+/*    */
+/*  	i=0 ;  */
+	
+
 	while(last_sample < 10000 && i < num_song_markers) {
 	    first_sample = last_sample ;
 	    i++ ;
@@ -1410,10 +1462,11 @@ void split_audio_on_markers(GtkWidget * widget, gpointer data)
 		snprintf(filename,99,"track%d.cdda.wav",trackno) ;
 	    }
 
-	    save_as_wavfile(filename, first_sample, last_sample) ;
-	    printf("Save as wavfile %s %d->%d\n", filename, first_sample, last_sample) ;
-
-	    trackno++ ;
+	    if(last_sample-first_sample >= 10000) {
+		save_as_wavfile(filename, first_sample, last_sample) ;
+		printf("Save as wavfile %s %d->%d\n", filename, first_sample, last_sample) ;
+		trackno++ ;
+	    }
 
 	    first_sample=last_sample+1 ;
 	    i++ ;
@@ -1424,7 +1477,6 @@ void split_audio_on_markers(GtkWidget * widget, gpointer data)
 		last_sample = prefs.n_samples-1 ;
 	    }
 
-	    if(last_sample-first_sample < 10000) break ;
 	}
 
 	file_processing = FALSE;
@@ -1980,7 +2032,7 @@ void open_file_selection(GtkWidget * widget, gpointer data)
     }
 }
 
-void save_selection_as_encoded(int fmt, char *filename, char *filename_new, struct view *v)
+void save_selection_as_encoded(int fmt, char *filename, char *filename_new, struct view *v, char *trackname)
 {
     long total_samples;
     total_samples = v->selected_last_sample - v->selected_first_sample + 1;
@@ -1993,35 +2045,35 @@ void save_selection_as_encoded(int fmt, char *filename, char *filename_new, stru
     /* save part of file as encoded fmt - fmt,old file, new file, start sample, number of samples */
 
     encode(fmt, filename, filename_new, v->selected_first_sample,
-	   total_samples);
+	   total_samples, trackname);
 }
 
 void store_selected_filename_as_encoded(GtkFileSelection * selector,
 				    gpointer user_data)
 {
+    int enc_format = NULL ;
+    int l;
+    char trackname[1024] = "" ;
+
+    if (encoding_type == GWC_OGG) enc_format = OGG_FMT ;
+    if (encoding_type == GWC_MP3) enc_format = MP3_FMT ;
+    if (encoding_type == GWC_MP3_SIMPLE) enc_format = MP3_SIMPLE_FMT ;
+
     strncpy(save_selection_filename,
 	   gtk_file_selection_get_filename(GTK_FILE_SELECTION
 					   (file_selector)), PATH_MAX);
 
     gtk_widget_destroy(file_selector);
 
-    if (strcmp(save_selection_filename, wave_filename)) {
-	int l;
-
+    if(!prompt_user("Enter the trackname:", trackname, 1023)) {
 	l = strlen(save_selection_filename);
 
 	file_processing = TRUE;
-	if (encoding_type == GWC_OGG)
-	    save_selection_as_encoded(OGGFMT, wave_filename,
-				      save_selection_filename,
-				      &audio_view);
-	else
-	    save_selection_as_encoded(MP3FMT, wave_filename,
-				      save_selection_filename,
-				      &audio_view);
+
+	save_selection_as_encoded(enc_format, wave_filename,
+				  save_selection_filename,
+				  &audio_view, trackname);
 	file_processing = FALSE;
-    } else {
-	warning("Cannot save over the currently open file!");
     }
 }
 
@@ -2042,7 +2094,7 @@ void save_as_encoded()
 		/* make it a .ogg extension */
 		bcopy(".ogg", strrchr(tmppath, '.'), 4);
 	    } else {
-		/* make it a .ogg extension */
+		/* make it a .mp3 extension */
 		bcopy(".mp3", strrchr(tmppath, '.'), 4);
 	    }
 
@@ -2083,6 +2135,12 @@ void save_as_mp3_selection(GtkWidget * widget, gpointer data)
     save_as_encoded();
 }
 
+void save_as_mp3_simple_selection(GtkWidget * widget, gpointer data)
+{
+    encoding_type = GWC_MP3_SIMPLE;
+    save_as_encoded();
+}
+
 void save_as_selection(GtkWidget * widget, gpointer data)
 {
 
@@ -2136,6 +2194,9 @@ GnomeUIInfo file_menu[] = {
     GNOMEUIINFO_ITEM_NONE("Save Selection As...",
 		      "Saves the current selection to a new wavfile",
 		      save_as_selection),
+    GNOMEUIINFO_ITEM_NONE("Simple Encode Selection as MP3",
+		      "Saves the current selection to a new MP3 encoded format, simple options",
+		      save_as_mp3_simple_selection),
     GNOMEUIINFO_ITEM_NONE("Encode Selection as MP3",
 		      "Saves the current selection to a new MP3 encoded format",
 		      save_as_mp3_selection),
@@ -2288,6 +2349,8 @@ GnomeUIInfo settings_menu[] = {
 		 decrackle_set_preferences, NULL),
     GNOMEUIINFO_ITEM("Denoise", "Set denoise parameters",
 		 denoise_set_preferences, NULL),
+    GNOMEUIINFO_ITEM("MP3 Simple Settings", "Set MP3 Simple Encoding parameters",
+		 set_mp3_simple_encoding_preferences, NULL),
     GNOMEUIINFO_ITEM("MP3 Settings", "Set MP3 Encoding parameters",
 		 set_mp3_encoding_preferences, NULL),
     GNOMEUIINFO_ITEM("Ogg Settings", "Set Ogg Encoding parameters",
@@ -2905,6 +2968,7 @@ int main(int argc, char *argv[])
     /* load all encoding preferences on start */
     load_ogg_encoding_preferences();
     load_mp3_encoding_preferences();
+    load_mp3_simple_encoding_preferences();
 
     /* create a new window */
     /*   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);  */
