@@ -289,7 +289,7 @@ void cdrdao_toc_info(char *filename)
 		 }
 		 fprintf(toc, "      }\n   }\n");
 	      }
-	      fprintf(toc, "   FILE \"%s\" %ld %ld\n", wave_filename, start, end - start) ;
+	      fprintf(toc, "   FILE \"%s\" %ld %ld\n", wave_filename, start, end - start + 1) ;
 
 	   }
 	   fclose(toc);
