@@ -28,11 +28,11 @@
 	library, parts 1 and 2
 */
 
-static char rcsid[] = "$Id: torture.c,v 1.1.1.1 2002/09/08 04:03:51 welty Exp $";
+static char rcsid[] = "$Id: torture.c,v 1.6 1994/08/25 15:22:11 des Exp $";
 
 #include	<stdio.h>
-#include	"matrix2.h"
 #include	<math.h>
+#include	"matrix2.h"
 #include        "matlab.h"
 
 #define	errmesg(mesg)	printf("Error: %s error: line %d\n",mesg,__LINE__)
@@ -589,7 +589,7 @@ char	*argv[];
 	       v_norm2(z), MACHEPS);
     }
     /* modified Cholesky factorisation should be identical with Cholesky
-       factorisation provided the matrix is "sufficiently positive definite */
+       factorisation provided the matrix is "sufficiently positive definite" */
     mtrm_mlt(B,B,C);
     MCHfactor(C,MACHEPS);
     m_sub(A,C,C);
